# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest

from azure.cli.core.extension._resolve import (resolve_from_index, resolve_project_url_from_index,
                                               NoExtensionCandidatesError, _is_not_platform_specific,
                                               _is_greater_than_cur_version)
from . import IndexPatch, mock_ext


class TestResolveFromIndex(unittest.TestCase):

    def test_no_exts_in_index(self):
        name = 'myext'
        with IndexPatch({}), self.assertRaises(NoExtensionCandidatesError) as err:
            resolve_from_index(name)
        self.assertEqual(str(err.exception), "No extension found with name '{}'".format(name))

    def test_ext_not_in_index(self):
        name = 'an_extension_b'
        with IndexPatch({'an_extension_a': []}), self.assertRaises(NoExtensionCandidatesError) as err:
            resolve_from_index(name)
        self.assertEqual(str(err.exception), "No extension found with name '{}'".format(name))

    def test_ext_resolved(self):
        name = 'myext'
        index_data = {name: [mock_ext('myext-0.0.1-py2.py3-none-any.whl', '0.0.1')]}
        with IndexPatch(index_data):
            self.assertEqual(resolve_from_index(name)[0], index_data[name][0]['downloadUrl'])

    def test_ext_py3_resolved(self):
        name = 'myext'
        index_data = {'myext': [mock_ext('myext-0.0.1-py3-none-any.whl', '0.0.1')]}
        with IndexPatch(index_data):
            self.assertEqual(resolve_from_index(name)[0], index_data[name][0]['downloadUrl'])

    def test_filter_platform_wrong_abi(self):
        name = 'myext'
        # Should raise as wrong abi of cp33d
        with IndexPatch({'myext': [mock_ext('myext-0.0.1-py2.py3-cp33d-any.whl', '0.0.1')]}), self.assertRaises(NoExtensionCandidatesError):
            resolve_from_index(name)

    def test_filter_platform_wrong_platform(self):
        name = 'myext'
        # Should raise as we don't support platform specific whls in index
        with IndexPatch({'myext': [mock_ext('myext-0.0.1-py2.py3-none-x86_64.whl', '0.0.1')]}), self.assertRaises(NoExtensionCandidatesError):
            resolve_from_index(name)

    def test_filter_platform(self):
        name = 'myext'
        index_data = {'myext': [mock_ext('myext-0.0.1-py2.py3-none-any.whl', '0.0.1'), mock_ext('myext-0.0.2-py3-none-any.whl', '0.0.2')]}
        with IndexPatch(index_data):
            # Should choose the second one as py version is not considered platform specific.
            self.assertEqual(resolve_from_index(name)[0], index_data[name][1]['downloadUrl'])

    def test_filter_target_version(self):
        ext_name = 'hello'
        index_data = {
            ext_name: [
                mock_ext('hello-0.1.0-py3-none-any.whl', '0.1.0'),
                mock_ext('hello-0.2.0-py3-none-any.whl', '0.2.0')
            ]
        }

        # resolve okay
        with IndexPatch(index_data):
            self.assertEqual(resolve_from_index(ext_name, target_version='0.1.0')[0], index_data[ext_name][0]['downloadUrl'])
            self.assertEqual(resolve_from_index(ext_name, target_version='0.2.0')[0], index_data[ext_name][1]['downloadUrl'])

        with IndexPatch(index_data):
            with self.assertRaisesRegex(NoExtensionCandidatesError, 'Extension with version 0.3.0 not found'):
                resolve_from_index(ext_name, target_version='0.3.0')


class TestResolveFilters(unittest.TestCase):

    def test_platform_specific(self):
        self.assertTrue(_is_not_platform_specific(mock_ext('myext-0.0.1-py2.py3-none-any.whl')))

        self.assertTrue(_is_not_platform_specific(mock_ext('myext-0.0.1-py2-none-any.whl')))
        self.assertFalse(_is_not_platform_specific(mock_ext('myext-0.0.1-py3-cp33d-any.whl')))
        self.assertFalse(_is_not_platform_specific(mock_ext('myext-0.0.1-py3-none-x86_64.whl')))
        self.assertFalse(_is_not_platform_specific(mock_ext('myext-1.1.26.0-py2-none-linux_armv7l.whl')))

    def test_greater_than_current(self):
        self.assertIsNone(_is_greater_than_cur_version(None))
        self.assertIsNotNone(_is_greater_than_cur_version('0.0.1'))
        filter_func = _is_greater_than_cur_version('0.0.1')
        filter_func
        self.assertTrue(filter_func(mock_ext('myext-0.0.2-py2.py3-none-any.whl', '0.0.2')))
        self.assertTrue(filter_func(mock_ext('myext-0.0.1+dev-py2.py3-none-any.whl', '0.0.1+dev')))
        self.assertTrue(filter_func(mock_ext('myext-0.0.1.post1-py2.py3-none-any.whl', '0.0.1.post1')))

        self.assertFalse(filter_func(mock_ext('myext-0.0.1.pre1-py2.py3-none-any.whl', '0.0.1.pre1')))
        self.assertFalse(filter_func(mock_ext('myext-0.0.1-py2.py3-none-any.whl', '0.0.1')))


class TestResolveProjectUrlFromIndex(unittest.TestCase):

    def test_project_url_for_no_exts_in_index(self):
        name = 'myext'
        with IndexPatch({}), self.assertRaises(NoExtensionCandidatesError) as err:
            resolve_project_url_from_index(name)
        self.assertEqual(str(err.exception), "No extension found with name '{}'".format(name))

    def test_project_url_for_no_matching_ext_in_index(self):
        name = 'an_extension_b'
        with IndexPatch({'an_extension_a': []}), self.assertRaises(NoExtensionCandidatesError) as err:
            resolve_project_url_from_index(name)
        self.assertEqual(str(err.exception), "No extension found with name '{}'".format(name))

    def test_project_url_for_ext_resolved(self):
        name = 'myext'
        index_data = {name: [mock_ext('myext-0.0.1-py2.py3-none-any.whl', '0.0.1')]}
        with IndexPatch(index_data):
            self.assertEqual(resolve_project_url_from_index(name), index_data[name][0]['metadata']['extensions']['python.details']['project_urls']['Home'])


if __name__ == '__main__':
    unittest.main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
from typing import Any

from azure.core.configuration import Configuration
from azure.core.pipeline import policies
from azure.mgmt.core.policies import ARMHttpLoggingPolicy

from ._version import VERSION


class SubscriptionClientConfiguration(Configuration):
    """Configuration for SubscriptionClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param credential: Credential needed for the client to connect to Azure.
    :type credential: ~azure.core.credentials.TokenCredential
    """

    def __init__(
        self,
        credential,  # type: "TokenCredential"
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        if credential is None:
            raise ValueError("Parameter 'credential' must not be None.")
        super(SubscriptionClientConfiguration, self).__init__(**kwargs)

        self.credential = credential
        self.credential_scopes = kwargs.pop('credential_scopes', ['https://management.azure.com/.default'])
        kwargs.setdefault('sdk_moniker', 'azure-mgmt-resource/{}'.format(VERSION))
        self._configure(**kwargs)

    def _configure(
        self,
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        self.user_agent_policy = kwargs.get('user_agent_policy') or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get('headers_policy') or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get('proxy_policy') or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get('logging_policy') or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get('http_logging_policy') or ARMHttpLoggingPolicy(**kwargs)
        self.retry_policy = kwargs.get('retry_policy') or policies.RetryPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get('custom_hook_policy') or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get('redirect_policy') or policies.RedirectPolicy(**kwargs)
        self.authentication_policy = kwargs.get('authentication_policy')
        if self.credential and not self.authentication_policy:
            self.authentication_policy = policies.BearerTokenCredentialPolicy(self.credential, *self.credential_scopes, **kwargs)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

import mock

from knack.util import CLIError


class TestNetworkUnitTests(unittest.TestCase):
    def test_network_get_nic_ip_config(self):
        from azure.cli.command_modules.network.custom import _get_nic_ip_config

        # 1 -  Test that if ip_configurations property is null, error is thrown
        nic = mock.MagicMock()
        nic.ip_configurations = None
        with self.assertRaises(CLIError):
            _get_nic_ip_config(nic, 'test')

        def mock_ip_config(name, value):
            fake = mock.MagicMock()
            fake.name = name
            fake.value = value
            return fake

        nic = mock.MagicMock()
        nic.ip_configurations = [mock_ip_config('test1', '1'), mock_ip_config('test2', '2'),
                                 mock_ip_config('test3', '3')]
        # 2 - Test that if ip_configurations is not null but no match, error is thrown
        with self.assertRaises(CLIError):
            _get_nic_ip_config(nic, 'test4')

        # 3 - Test that match is returned
        self.assertEqual(_get_nic_ip_config(nic, 'test2').value, '2')

    def test_network_upsert(self):
        from azure.cli.core.commands import upsert_to_collection

        obj1 = mock.MagicMock()
        obj1.key = 'object1'
        obj1.value = 'cat'

        obj2 = mock.MagicMock()
        obj2.key = 'object2'
        obj2.value = 'dog'

        # 1 - verify upsert to a null collection
        parent_with_null_collection = mock.MagicMock()
        parent_with_null_collection.collection = None
        upsert_to_collection(parent_with_null_collection, 'collection', obj1, 'key')
        result = parent_with_null_collection.collection
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].value, 'cat')

        # 2 - verify upsert to an empty collection
        parent_with_empty_collection = mock.MagicMock()
        parent_with_empty_collection.collection = []
        upsert_to_collection(parent_with_empty_collection, 'collection', obj1, 'key')
        result = parent_with_empty_collection.collection
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].value, 'cat')

        # 3 - verify can add more than one
        upsert_to_collection(parent_with_empty_collection, 'collection', obj2, 'key')
        result = parent_with_empty_collection.collection
        self.assertEqual(len(result), 2)
        self.assertEqual(result[1].value, 'dog')

        # 4 - verify update to existing collection
        obj2.value = 'noodle'
        upsert_to_collection(parent_with_empty_collection, 'collection', obj2, 'key')
        result = parent_with_empty_collection.collection
        self.assertEqual(len(result), 2)
        self.assertEqual(result[1].value, 'noodle')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer
from azure.cli.command_modules.ams._test_utils import _get_test_data_file


class AmsStreamingPolicyTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'West US')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS'
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --no-encryption-protocols {protocols}', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('az ams streaming-policy show -a {amsname} -n {streamingPolicyName} -g {rg}', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('noEncryption.enabledProtocols.hls', True)
        ])

        list = self.cmd('az ams streaming-policy list -a {amsname} -g {rg}').get_output_in_json()
        assert len(list) > 0

        self.cmd('az ams streaming-policy delete -n {streamingPolicyName} -a {amsname} -g {rg}')

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy_envelope(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'eastus2'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'East US 2')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS Dash',
            'urlTemplate': 'https://xyz.foo.bar',
            'label': 'label'
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --envelope-protocols {protocols} --envelope-template {urlTemplate} --envelope-default-key-label {label}', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('envelopeEncryption.enabledProtocols.hls', True),
            self.check('envelopeEncryption.enabledProtocols.dash', True),
            self.check('envelopeEncryption.contentKeys.defaultKey.label', '{label}'),
            self.check('envelopeEncryption.customKeyAcquisitionUrlTemplate', '{urlTemplate}')
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy_cenc(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus2'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'West US 2')
        ])

        policy_option_name1 = self.create_random_name(prefix='pon', length=12)
        policy_option_name2 = self.create_random_name(prefix='pon', length=12)

        self.kwargs.update({
            'description': 'ExampleDescription',
            'playReadyPath': '@' + _get_test_data_file('validPlayReadyTemplate.json'),
            'policyNameFromKeyToTrackMappings1': 'ckp',
            'policyNameFromKeyToTrackMappings2': 'demoPolicy15',
            'policyOptionName1': policy_option_name1,
            'policyOptionName2': policy_option_name2
        })

        self.cmd('az ams content-key-policy create -a {amsname} -n {policyNameFromKeyToTrackMappings1} -g {rg} --description {description} --open-restriction --play-ready-template "{playReadyPath}"  --policy-option-name {policyOptionName1}', checks=[
            self.check('name', '{policyNameFromKeyToTrackMappings1}')
        ])

        self.cmd('az ams content-key-policy create -a {amsname} -n {policyNameFromKeyToTrackMappings2} -g {rg} --description {description} --open-restriction --clear-key-configuration --policy-option-name {policyOptionName2}', checks=[
            self.check('name', '{policyNameFromKeyToTrackMappings2}')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS SmoothStreaming',
            'clearTracks': '@' + _get_test_data_file('clearTracks.json'),
            'keyToTrackMappings': '@' + _get_test_data_file('keyToTrackMappings.json'),
            'label': 'label',
            'playReadyUrlTemplate': 'https://playReadyTemplate.foo.bar',
            'playReadyAttributes': 'awesomeAttributes',
            'widevineUrlTemplate': 'https://widevineTemplate.foo.bar'
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --cenc-protocols {protocols} --cenc-clear-tracks "{clearTracks}" --cenc-key-to-track-mappings "{keyToTrackMappings}" --cenc-default-key-label {label} --cenc-default-key-policy-name {policyNameFromKeyToTrackMappings1} --cenc-play-ready-template {playReadyUrlTemplate} --cenc-play-ready-attributes {playReadyAttributes} --cenc-widevine-template {widevineUrlTemplate}', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('commonEncryptionCenc.enabledProtocols.hls', True),
            self.check('commonEncryptionCenc.enabledProtocols.smoothStreaming', True),
            self.check('commonEncryptionCenc.contentKeys.defaultKey.label', '{label}'),
            self.check('commonEncryptionCenc.drm.playReady.customLicenseAcquisitionUrlTemplate', '{playReadyUrlTemplate}'),
            self.check('commonEncryptionCenc.drm.playReady.playReadyCustomAttributes', '{playReadyAttributes}'),
            self.check('commonEncryptionCenc.drm.widevine.customLicenseAcquisitionUrlTemplate', '{widevineUrlTemplate}'),
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy_cenc_default_drm(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus2'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'West US 2')
        ])

        policy_option_name1 = self.create_random_name(prefix='pon', length=12)
        policy_option_name2 = self.create_random_name(prefix='pon', length=12)

        self.kwargs.update({
            'description': 'ExampleDescription',
            'playReadyPath': '@' + _get_test_data_file('validPlayReadyTemplate.json'),
            'policyNameFromKeyToTrackMappings1': 'ckp',
            'policyNameFromKeyToTrackMappings2': 'demoPolicy15',
            'policyOptionName1': policy_option_name1,
            'policyOptionName2': policy_option_name2
        })

        self.cmd('az ams content-key-policy create -a {amsname} -n {policyNameFromKeyToTrackMappings1} -g {rg} --description {description} --open-restriction --play-ready-template "{playReadyPath}"  --policy-option-name {policyOptionName1}', checks=[
            self.check('name', '{policyNameFromKeyToTrackMappings1}')
        ])

        self.cmd('az ams content-key-policy create -a {amsname} -n {policyNameFromKeyToTrackMappings2} -g {rg} --description {description} --open-restriction --clear-key-configuration --policy-option-name {policyOptionName2}', checks=[
            self.check('name', '{policyNameFromKeyToTrackMappings2}')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS SmoothStreaming',
            'clearTracks': '@' + _get_test_data_file('clearTracks.json'),
            'keyToTrackMappings': '@' + _get_test_data_file('keyToTrackMappings.json'),
            'label': 'label',
            'playReadyUrlTemplate': 'https://playReadyTemplate.foo.bar',
            'playReadyAttributes': 'awesomeAttributes'
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --cenc-protocols {protocols} --cenc-clear-tracks "{clearTracks}" --cenc-key-to-track-mappings "{keyToTrackMappings}" --cenc-default-key-label {label}', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('commonEncryptionCenc.enabledProtocols.hls', True),
            self.check('commonEncryptionCenc.enabledProtocols.smoothStreaming', True),
            self.check('commonEncryptionCenc.contentKeys.defaultKey.label', '{label}'),
            self.check('commonEncryptionCenc.drm.playReady.customLicenseAcquisitionUrlTemplate', None),
            self.check('commonEncryptionCenc.drm.playReady.playReadyCustomAttributes', None),
            self.check('commonEncryptionCenc.drm.widevine.customLicenseAcquisitionUrlTemplate', None),
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy_cenc_disable_widevine(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus2'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'West US 2')
        ])

        policy_option_name1 = self.create_random_name(prefix='pon', length=12)
        policy_option_name2 = self.create_random_name(prefix='pon', length=12)

        self.kwargs.update({
            'description': 'ExampleDescription',
            'playReadyPath': '@' + _get_test_data_file('validPlayReadyTemplate.json'),
            'policyNameFromKeyToTrackMappings1': 'ckp',
            'policyNameFromKeyToTrackMappings2': 'demoPolicy15',
            'policyOptionName1': policy_option_name1,
            'policyOptionName2': policy_option_name2
        })

        self.cmd('az ams content-key-policy create -a {amsname} -n {policyNameFromKeyToTrackMappings1} -g {rg} --description {description} --open-restriction --play-ready-template "{playReadyPath}"  --policy-option-name {policyOptionName1}', checks=[
            self.check('name', '{policyNameFromKeyToTrackMappings1}')
        ])

        self.cmd('az ams content-key-policy create -a {amsname} -n {policyNameFromKeyToTrackMappings2} -g {rg} --description {description} --open-restriction --clear-key-configuration --policy-option-name {policyOptionName2}', checks=[
            self.check('name', '{policyNameFromKeyToTrackMappings2}')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS SmoothStreaming',
            'clearTracks': '@' + _get_test_data_file('clearTracks.json'),
            'keyToTrackMappings': '@' + _get_test_data_file('keyToTrackMappings.json'),
            'label': 'label',
            'playReadyUrlTemplate': 'https://playReadyTemplate.foo.bar',
            'playReadyAttributes': 'awesomeAttributes'
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --cenc-protocols {protocols} --cenc-clear-tracks "{clearTracks}" --cenc-key-to-track-mappings "{keyToTrackMappings}" --cenc-default-key-label {label} --cenc-disable-widevine', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('commonEncryptionCenc.enabledProtocols.hls', True),
            self.check('commonEncryptionCenc.enabledProtocols.smoothStreaming', True),
            self.check('commonEncryptionCenc.contentKeys.defaultKey.label', '{label}'),
            self.check('commonEncryptionCenc.drm.playReady.customLicenseAcquisitionUrlTemplate', None),
            self.check('commonEncryptionCenc.drm.playReady.playReadyCustomAttributes', None),
            self.check('commonEncryptionCenc.drm.widevine.customLicenseAcquisitionUrlTemplate', None),
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy_cbcs(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'canadacentral'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Canada Central')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS SmoothStreaming Dash',
            'label': 'label',
            'urlTemplate': 'https://xyz.foo.bar',
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --cbcs-protocols {protocols} --cbcs-fair-play-template {urlTemplate} --cbcs-default-key-label {label} --cbcs-fair-play-allow-persistent-license', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('commonEncryptionCbcs.enabledProtocols.hls', True),
            self.check('commonEncryptionCbcs.enabledProtocols.smoothStreaming', True),
            self.check('commonEncryptionCbcs.enabledProtocols.dash', True),
            self.check('commonEncryptionCbcs.contentKeys.defaultKey.label', '{label}'),
            self.check('commonEncryptionCbcs.drm.fairPlay.customLicenseAcquisitionUrlTemplate', '{urlTemplate}'),
            self.check('commonEncryptionCbcs.drm.fairPlay.allowPersistentLicense', True),
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_streaming_policy_cbcs_default_drm(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'canadacentral'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Canada Central')
        ])

        streamingPolicyName = self.create_random_name(prefix='spn', length=10)

        self.kwargs.update({
            'streamingPolicyName': streamingPolicyName,
            'protocols': 'HLS SmoothStreaming Dash',
            'label': 'label',
            'urlTemplate': 'https://xyz.foo.bar',
        })

        self.cmd('az ams streaming-policy create -a {amsname} -n {streamingPolicyName} -g {rg} --cbcs-protocols {protocols} --cbcs-default-key-label {label} ', checks=[
            self.check('name', '{streamingPolicyName}'),
            self.check('commonEncryptionCbcs.enabledProtocols.hls', True),
            self.check('commonEncryptionCbcs.enabledProtocols.smoothStreaming', True),
            self.check('commonEncryptionCbcs.enabledProtocols.dash', True),
            self.check('commonEncryptionCbcs.contentKeys.defaultKey.label', '{label}'),
            self.check('commonEncryptionCbcs.drm.fairPlay.customLicenseAcquisitionUrlTemplate', None),
            self.check('commonEncryptionCbcs.drm.fairPlay.allowPersistentLicense', False),
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ResourceGroupPreparer, JMESPathCheck
from azure.cli.testsdk import ScenarioTest
from azure.mgmt.cdn.models import (SkuName,
                                   CdnEndpoint,
                                   EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink,
                                   PolicyMode,
                                   ActionType,
                                   ErrorResponseException)
from .scenario_mixin import CdnScenarioMixin
from base64 import b64encode
from knack.util import CLIError


class CdnWafPolicyScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    def test_cdn_waf_policy_crud(self, resource_group):

        self.kwargs.update({
            'resource_group': resource_group,
            'policy': 'policy123',
            'custom_rule_1': 'customrule1',
            'custom_rule_2': 'customrule2',
            'rate_limit_rule_1': 'ratelimitrule1',
            'rate_limit_rule_2': 'ratelimitrule2',
            'rule_set_type': 'DefaultRuleSet',
            'rule_set_version': '1.0',
            'rule_group': 'SQLI',
            'redirect_url': 'https://example.com',
            'block_response_body': b64encode('<html><body>example body</body></html>'.encode('utf-8')).decode('utf-8'),
            'block_response_status_code': 429,
        })

        # Create, show, and list empty policy.
        checks = [JMESPathCheck('name', self.kwargs['policy']),
                  JMESPathCheck('sku.name', SkuName.standard_microsoft.value),
                  JMESPathCheck('customRules.rules', []),
                  JMESPathCheck('rateLimitRules.rules', []),
                  JMESPathCheck('managedRules.managedRuleSets', []),
                  JMESPathCheck('policySettings.enabledState', 'Enabled'),
                  JMESPathCheck('policySettings.defaultRedirectUrl', self.kwargs['redirect_url']),
                  JMESPathCheck('endpointLinks', []),
                  JMESPathCheck('tags.foo', 'bar')]
        list_checks = [JMESPathCheck('length(@)', 1),
                       JMESPathCheck('@[0].name', self.kwargs['policy']),
                       JMESPathCheck('@[0].sku.name', SkuName.standard_microsoft.value),
                       JMESPathCheck('@[0].customRules.rules', []),
                       JMESPathCheck('@[0].rateLimitRules.rules', []),
                       JMESPathCheck('@[0].managedRules.managedRuleSets', []),
                       JMESPathCheck('@[0].policySettings.enabledState', 'Enabled'),
                       JMESPathCheck('@[0].endpointLinks', []),
                       JMESPathCheck('@[0].tags.foo', 'bar')]
        with self.assertRaises(SystemExit):  # exits with code 3 due to missing resource
            self.cmd('cdn waf policy show -g {resource_group} -n {policy}')
        self.cmd(
            'cdn waf policy set -g {resource_group} --name {policy} --redirect-url "{redirect_url}" --tags="foo=bar"', checks=checks)
        self.cmd('cdn waf policy show -g {resource_group} -n {policy}', checks=checks)
        self.cmd('cdn waf policy list -g {resource_group}', checks=list_checks)

        # Add, show, and list managed rule set.
        checks = [JMESPathCheck('ruleSetType', self.kwargs['rule_set_type']),
                  JMESPathCheck('ruleSetVersion', self.kwargs['rule_set_version']),
                  JMESPathCheck('managedRuleOverrides', None)]
        list_checks = [JMESPathCheck('length(@)', 1),
                       JMESPathCheck('@[0]ruleSetType', self.kwargs['rule_set_type']),
                       JMESPathCheck('@[0]ruleSetVersion', self.kwargs['rule_set_version']),
                       JMESPathCheck('@[0]managedRuleOverrides', None)]
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy managed-rule-set show -g {resource_group} --policy-name {policy} '
                     '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version}')
        self.cmd('cdn waf policy managed-rule-set add -g {resource_group} --policy-name {policy} '
                 '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version}',
                 checks=checks)
        self.cmd('cdn waf policy managed-rule-set show -g {resource_group} --policy-name {policy} '
                 '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version}',
                 checks=checks)

        # Set, show, and list rule group override.
        checks = [JMESPathCheck('ruleGroupName', self.kwargs['rule_group']),
                  JMESPathCheck('rules[0].ruleId', '942440'),
                  JMESPathCheck('rules[0].action', 'Redirect'),
                  JMESPathCheck('rules[0].enabledState', 'Enabled'),
                  JMESPathCheck('rules[1].ruleId', '942120'),
                  JMESPathCheck('rules[1].action', 'Block'),
                  JMESPathCheck('rules[1].enabledState', 'Disabled'),
                  JMESPathCheck('rules[2].ruleId', '942100'),
                  JMESPathCheck('rules[2].action', 'Block'),
                  JMESPathCheck('rules[2].enabledState', 'Disabled')]
        list_checks = [JMESPathCheck('length(@)', 1),
                       JMESPathCheck('@[0].ruleGroupName', self.kwargs['rule_group']),
                       JMESPathCheck('@[0].rules[0].ruleId', '942440'),
                       JMESPathCheck('@[0].rules[0].action', 'Redirect'),
                       JMESPathCheck('@[0].rules[0].enabledState', 'Enabled'),
                       JMESPathCheck('@[0].rules[1].ruleId', '942120'),
                       JMESPathCheck('@[0].rules[1].action', 'Block'),
                       JMESPathCheck('@[0].rules[1].enabledState', 'Disabled'),
                       JMESPathCheck('@[0].rules[2].ruleId', '942100'),
                       JMESPathCheck('@[0].rules[2].action', 'Block'),
                       JMESPathCheck('@[0].rules[2].enabledState', 'Disabled')]
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy managed-rule-set rule-group-override show -g {resource_group} '
                     '--policy-name {policy} --rule-set-type {rule_set_type} '
                     '--rule-set-version {rule_set_version} -n {rule_group}')
        self.cmd('cdn waf policy managed-rule-set rule-group-override set -g {resource_group} --policy-name {policy} '
                 '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version} -n {rule_group} '
                 '-r id=942440 action=Redirect enabled=Enabled -r id=942120 -r id=942100')
        self.cmd('cdn waf policy managed-rule-set rule-group-override show -g {resource_group} '
                 '--policy-name {policy} --rule-set-type {rule_set_type} '
                 '--rule-set-version {rule_set_version} -n {rule_group}',
                 checks=checks)
        self.cmd('cdn waf policy managed-rule-set rule-group-override list -g {resource_group} '
                 '--policy-name {policy} --rule-set-type {rule_set_type} '
                 '--rule-set-version {rule_set_version}',
                 checks=list_checks)

        # Set, show, and list custom rules.
        checks = [JMESPathCheck('name', self.kwargs['custom_rule_1']),
                  JMESPathCheck('action', ActionType.block.value),
                  JMESPathCheck('priority', 100),
                  JMESPathCheck('length(matchConditions)', 1),
                  JMESPathCheck('matchConditions[0].matchVariable', 'RequestMethod'),
                  JMESPathCheck('matchConditions[0].operator', 'Equal'),
                  JMESPathCheck('matchConditions[0].negateCondition', True),
                  JMESPathCheck('length(matchConditions[0].matchValue)', 2),
                  JMESPathCheck('matchConditions[0].matchValue[0]', 'GET'),
                  JMESPathCheck('matchConditions[0].matchValue[1]', 'HEAD')]
        checks2 = [JMESPathCheck('name', self.kwargs['custom_rule_2']),
                   JMESPathCheck('action', ActionType.redirect.value),
                   JMESPathCheck('priority', 200),
                   JMESPathCheck('length(matchConditions)', 2),
                   JMESPathCheck('matchConditions[0].matchVariable', 'RequestUri'),
                   JMESPathCheck('matchConditions[0].operator', 'Contains'),
                   JMESPathCheck('matchConditions[0].negateCondition', False),
                   JMESPathCheck('length(matchConditions[0].matchValue)', 1),
                   JMESPathCheck('matchConditions[0].matchValue[0]', '..'),
                   JMESPathCheck('matchConditions[1].matchVariable', 'QueryString'),
                   JMESPathCheck('matchConditions[1].operator', 'Contains'),
                   JMESPathCheck('matchConditions[1].negateCondition', False),
                   JMESPathCheck('length(matchConditions[1].matchValue)', 1),
                   JMESPathCheck('matchConditions[1].matchValue[0]', ' ')]
        list_checks = [JMESPathCheck('@[0].name', self.kwargs['custom_rule_1']),
                       JMESPathCheck('@[0].action', ActionType.block.value),
                       JMESPathCheck('@[0].priority', 100),
                       JMESPathCheck('length(@[0].matchConditions)', 1),
                       JMESPathCheck('@[0].matchConditions[0].matchVariable', 'RequestMethod'),
                       JMESPathCheck('@[0].matchConditions[0].operator', 'Equal'),
                       JMESPathCheck('@[0].matchConditions[0].negateCondition', True),
                       JMESPathCheck('length(@[0].matchConditions[0].matchValue)', 2),
                       JMESPathCheck('@[0].matchConditions[0].matchValue[0]', 'GET'),
                       JMESPathCheck('@[0].matchConditions[0].matchValue[1]', 'HEAD'),
                       JMESPathCheck('@[1].name', self.kwargs['custom_rule_2']),
                       JMESPathCheck('@[1].action', ActionType.redirect.value),
                       JMESPathCheck('@[1].priority', 200),
                       JMESPathCheck('length(@[1].matchConditions)', 2),
                       JMESPathCheck('@[1].matchConditions[0].matchVariable', 'RequestUri'),
                       JMESPathCheck('@[1].matchConditions[0].operator', 'Contains'),
                       JMESPathCheck('@[1].matchConditions[0].negateCondition', False),
                       JMESPathCheck('length(@[1].matchConditions[0].matchValue)', 1),
                       JMESPathCheck('@[1].matchConditions[0].matchValue[0]', '..'),
                       JMESPathCheck('@[1].matchConditions[1].matchVariable', 'QueryString'),
                       JMESPathCheck('@[1].matchConditions[1].operator', 'Contains'),
                       JMESPathCheck('@[1].matchConditions[1].negateCondition', False),
                       JMESPathCheck('length(@[1].matchConditions[1].matchValue)', 1),
                       JMESPathCheck('@[1].matchConditions[1].matchValue[0]', ' ')]
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy custom-rule show -g {resource_group} --policy-name {policy} -n {custom_rule_1}')
        self.cmd('cdn waf policy custom-rule set -g {resource_group} --policy-name {policy} -n {custom_rule_1} '
                 '--action Block --priority 100 --match-condition '
                 'match-variable=RequestMethod operator=Equal negate=true match-value=GET match-value=HEAD',
                 checks=checks)
        self.cmd('cdn waf policy custom-rule show -g {resource_group} --policy-name {policy} -n {custom_rule_1}',
                 checks=checks)
        self.cmd('cdn waf policy custom-rule set -g {resource_group} --policy-name {policy} -n {custom_rule_2} '
                 '--action Redirect --priority 200 '
                 '-m match-variable=RequestUri operator=Contains match-value=.. '
                 '-m match-variable=QueryString operator=Contains "match-value= "',
                 checks=checks2)
        self.cmd('cdn waf policy custom-rule show -g {resource_group} --policy-name {policy} -n {custom_rule_2}',
                 checks=checks2)
        self.cmd('cdn waf policy custom-rule list -g {resource_group} --policy-name {policy}', checks=list_checks)

        # Set, show, and list rate limit rules.
        checks = [JMESPathCheck('name', self.kwargs['rate_limit_rule_1']),
                  JMESPathCheck('action', ActionType.block.value),
                  JMESPathCheck('priority', 100),
                  JMESPathCheck('rateLimitThreshold', 100),
                  JMESPathCheck('rateLimitDurationInMinutes', 1),
                  JMESPathCheck('length(matchConditions)', 1),
                  JMESPathCheck('matchConditions[0].matchVariable', 'RequestMethod'),
                  JMESPathCheck('matchConditions[0].operator', 'Equal'),
                  JMESPathCheck('matchConditions[0].negateCondition', True),
                  JMESPathCheck('length(matchConditions[0].matchValue)', 2),
                  JMESPathCheck('matchConditions[0].matchValue[0]', 'GET'),
                  JMESPathCheck('matchConditions[0].matchValue[1]', 'HEAD')]
        checks2 = [JMESPathCheck('name', self.kwargs['rate_limit_rule_2']),
                   JMESPathCheck('action', ActionType.redirect.value),
                   JMESPathCheck('priority', 200),
                   JMESPathCheck('rateLimitThreshold', 100),
                   JMESPathCheck('rateLimitDurationInMinutes', 5),
                   JMESPathCheck('length(matchConditions)', 2),
                   JMESPathCheck('matchConditions[0].matchVariable', 'RequestMethod'),
                   JMESPathCheck('matchConditions[0].operator', 'Equal'),
                   JMESPathCheck('matchConditions[0].negateCondition', False),
                   JMESPathCheck('length(matchConditions[0].matchValue)', 1),
                   JMESPathCheck('matchConditions[0].matchValue[0]', 'PUT'),
                   JMESPathCheck('matchConditions[1].matchVariable', 'RequestUri'),
                   JMESPathCheck('matchConditions[1].operator', 'Contains'),
                   JMESPathCheck('matchConditions[1].negateCondition', False),
                   JMESPathCheck('length(matchConditions[1].matchValue)', 1),
                   JMESPathCheck('matchConditions[1].matchValue[0]', '/expensive/resource/')]
        list_checks = [JMESPathCheck('@[0].name', self.kwargs['rate_limit_rule_1']),
                       JMESPathCheck('@[0].action', ActionType.block.value),
                       JMESPathCheck('@[0].priority', 100),
                       JMESPathCheck('@[0].rateLimitThreshold', 100),
                       JMESPathCheck('@[0].rateLimitDurationInMinutes', 1),
                       JMESPathCheck('length(@[0].matchConditions)', 1),
                       JMESPathCheck('@[0].matchConditions[0].matchVariable', 'RequestMethod'),
                       JMESPathCheck('@[0].matchConditions[0].operator', 'Equal'),
                       JMESPathCheck('@[0].matchConditions[0].negateCondition', True),
                       JMESPathCheck('length(@[0].matchConditions[0].matchValue)', 2),
                       JMESPathCheck('@[0].matchConditions[0].matchValue[0]', 'GET'),
                       JMESPathCheck('@[0].matchConditions[0].matchValue[1]', 'HEAD'),
                       JMESPathCheck('@[1].name', self.kwargs['rate_limit_rule_2']),
                       JMESPathCheck('@[1].action', ActionType.redirect.value),
                       JMESPathCheck('@[1].priority', 200),
                       JMESPathCheck('@[1].rateLimitThreshold', 100),
                       JMESPathCheck('@[1].rateLimitDurationInMinutes', 5),
                       JMESPathCheck('length(@[1].matchConditions)', 2),
                       JMESPathCheck('@[1].matchConditions[0].matchVariable', 'RequestMethod'),
                       JMESPathCheck('@[1].matchConditions[0].operator', 'Equal'),
                       JMESPathCheck('@[1].matchConditions[0].negateCondition', False),
                       JMESPathCheck('length(@[1].matchConditions[0].matchValue)', 1),
                       JMESPathCheck('@[1].matchConditions[0].matchValue[0]', 'PUT'),
                       JMESPathCheck('@[1].matchConditions[1].matchVariable', 'RequestUri'),
                       JMESPathCheck('@[1].matchConditions[1].operator', 'Contains'),
                       JMESPathCheck('@[1].matchConditions[1].negateCondition', False),
                       JMESPathCheck('length(@[1].matchConditions[1].matchValue)', 1),
                       JMESPathCheck('@[1].matchConditions[1].matchValue[0]', '/expensive/resource/')]
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy rate-limit-rule show -g {resource_group} --policy-name {policy} -n {rate_limit_rule_1}')
        self.cmd('cdn waf policy rate-limit-rule set -g {resource_group} --policy-name {policy} -n {rate_limit_rule_1} '
                 '--action Block --priority 100 --duration 1 --request-threshold 100 --match-condition '
                 'match-variable=RequestMethod operator=Equal negate=true match-value=GET match-value=HEAD',
                 checks=checks)
        self.cmd('cdn waf policy rate-limit-rule show -g {resource_group} --policy-name {policy} -n {rate_limit_rule_1}',
                 checks=checks)
        self.cmd('cdn waf policy rate-limit-rule set -g {resource_group} --policy-name {policy} -n {rate_limit_rule_2} '
                 '--action Redirect --priority 200 --duration 5 --request-threshold 100 '
                 '-m match-variable=RequestMethod operator=Equal match-value=PUT '
                 '-m match-variable=RequestUri operator=Contains match-value=/expensive/resource/',
                 checks=checks2)
        self.cmd('cdn waf policy rate-limit-rule show -g {resource_group} --policy-name {policy} -n {rate_limit_rule_2}',
                 checks=checks2)
        self.cmd('cdn waf policy rate-limit-rule list -g {resource_group} --policy-name {policy}',
                 checks=list_checks)

        # Update and show policy with custom, rate limit, and managed rules set.
        checks = [JMESPathCheck('name', self.kwargs['policy']),
                  JMESPathCheck('sku.name', SkuName.standard_microsoft.value),
                  JMESPathCheck('length(customRules.rules)', 2),
                  JMESPathCheck('length(rateLimitRules.rules)', 2),
                  JMESPathCheck('length(managedRules.managedRuleSets)', 1),
                  JMESPathCheck('policySettings.enabledState', 'Disabled'),
                  JMESPathCheck('policySettings.defaultRedirectUrl', self.kwargs['redirect_url']),
                  JMESPathCheck('policySettings.defaultCustomBlockResponseStatusCode',
                                self.kwargs['block_response_status_code']),
                  JMESPathCheck('policySettings.defaultCustomBlockResponseBody', self.kwargs['block_response_body']),
                  JMESPathCheck('policySettings.mode', PolicyMode.prevention.value),
                  JMESPathCheck('tags', {}),
                  JMESPathCheck('endpointLinks', [])]
        self.cmd('cdn waf policy set -g {resource_group} '
                 '--name {policy} '
                 '--sku Standard_Microsoft '
                 '--mode Prevention '
                 '--block-response-body {block_response_body} '
                 '--block-response-status-code {block_response_status_code} '
                 '--redirect-url {redirect_url} '
                 '--tags="" '
                 '--disabled',
                 checks=checks)
        self.cmd('cdn waf policy show -g {resource_group} -n {policy}', checks=checks)

        # Remove managed rule group override
        self.cmd('cdn waf policy managed-rule-set rule-group-override delete -y -g {resource_group} --policy-name {policy} '
                 '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version} -n {rule_group}')
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy managed-rule-set rule-group-override show -g {resource_group} '
                     '--policy-name {policy} --rule-set-type {rule_set_type} '
                     '--rule-set-version {rule_set_version} -n {rule_group}')

        # Remove managed rule set
        self.cmd('cdn waf policy managed-rule-set remove -y -g {resource_group} --policy-name {policy} '
                 '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version}')
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy managed-rule-set show -g {resource_group} --policy-name {policy} '
                     '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version}')

        # Remove custom rule
        self.cmd('cdn waf policy custom-rule delete -y -g {resource_group} --policy-name {policy} -n {custom_rule_1}')
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy custom-rule show -g {resource_group} --policy-name {policy} -n {custom_rule_1}')

        # Remove rate limit rule
        self.cmd('cdn waf policy rate-limit-rule delete -y -g {resource_group} --policy-name {policy} -n {rate_limit_rule_1}')
        with self.assertRaises(CLIError):
            self.cmd('cdn waf policy rate-limit-rule show -g {resource_group} --policy-name {policy} -n {rate_limit_rule_1}')

        # Delete policy
        self.cmd('cdn waf policy delete -y -g {resource_group} -n {policy}')


class CdnWafEndpointLinkScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_cdn_domain')
    def test_cdn_waf_endpoint_link(self, resource_group):

        policy = 'wafpolicy1'
        profile = 'ps1'
        endpoint1 = self.create_random_name(prefix='ep1', length=13)
        endpoint2 = self.create_random_name(prefix='ep2', length=13)
        policy_id = self.policy_id(resource_group, policy)

        self.kwargs.update({
            'subscription_id': self.get_subscription_id(),
            'resource_group': resource_group,
            'profile': 'ps1',
            'endpoint1': endpoint1,
            'endpoint2': endpoint2,
            'policy': policy,
            'policy_id': policy_id,
        })

        self.cmd('cdn profile create -g {resource_group} -n {profile} --sku=Standard_Microsoft --location=westus')

        # Create WAF policy and linked endpoint.
        policy_checks = [JMESPathCheck('length(endpointLinks)', 0),
                         JMESPathCheck('id', policy_id, case_sensitive=False)]
        self.cmd('cdn waf policy set -g {resource_group} --name {policy}', checks=policy_checks)

        # Create the endpoint.
        endpoint_checks = [JMESPathCheck('webApplicationFirewallPolicyLink', None)]
        self.cmd('cdn endpoint create -g {resource_group} '
                 '--origin www.test.com '
                 '--profile-name {profile} '
                 '-n {endpoint1} ',
                 checks=endpoint_checks)
        with self.assertRaises(CLIError):
            self.cmd('cdn endpoint waf policy show -g {resource_group} --profile-name {profile} --endpoint-name {endpoint1}')

        # Link the endpoint.
        # link_checks = [JMESPathCheck('id', policy_id, case_sensitive=False)]
        endpoint_checks = [JMESPathCheck('webApplicationFirewallPolicyLink.id', policy_id, case_sensitive=False)]
        policy_checks = [JMESPathCheck('length(endpointLinks)', 1),
                         JMESPathCheck('endpointLinks[0].id', self.endpoint_id(resource_group, profile, endpoint1), case_sensitive=False)]
        self.cmd('cdn endpoint waf policy set -g {resource_group} --profile-name {profile} --endpoint-name {endpoint1} '
                 '--waf-policy-subscription-id {subscription_id} --waf-policy-resource-group-name {resource_group} '
                 '--waf-policy-name {policy}')
        # self.cmd('cdn endpoint waf policy show -g {resource_group} --profile-name {profile} --endpoint-name {endpoint1}',
        #          checks=link_checks)
        self.cmd('cdn endpoint show -g {resource_group} '
                 '--profile-name {profile} '
                 '-n {endpoint1}',)
        self.cmd('cdn waf policy show -g {resource_group} -n {policy}')

        # Create and link the second endpoint.
        policy_checks = [JMESPathCheck('length(endpointLinks)', 2),
                         JMESPathCheck('endpointLinks[0].id', self.endpoint_id(resource_group, profile, endpoint1), case_sensitive=False),
                         JMESPathCheck('endpointLinks[1].id', self.endpoint_id(resource_group, profile, endpoint2), case_sensitive=False)]
        self.cmd('cdn endpoint create -g {resource_group} '
                 '--origin www.test.com '
                 '--profile-name {profile} '
                 '-n {endpoint2}')
        self.cmd('cdn endpoint waf policy set -g {resource_group} '
                 '--profile-name {profile} '
                 '--endpoint-name {endpoint2} '
                 '--waf-policy-id {policy_id}')
        # self.cmd('cdn endpoint waf policy show -g {resource_group} --profile-name {profile} --endpoint-name {endpoint2}',
        #          checks=link_checks)
        self.cmd('cdn endpoint show -g {resource_group} '
                 '--profile-name {profile} '
                 '-n {endpoint2}',
                 )
        self.cmd('cdn waf policy show -g {resource_group} -n {policy}')

        # Remove both endpoint links
        policy_checks = [JMESPathCheck('length(endpointLinks)', 0)]
        endpoint_checks = [JMESPathCheck('webApplicationFirewallPolicyLink', None)]
        self.cmd('cdn endpoint waf policy remove -y -g {resource_group} '
                 '--profile-name {profile} '
                 '--endpoint-name {endpoint1}')
        with self.assertRaises(CLIError):
            self.cmd('cdn endpoint waf policy show -g {resource_group} --profile-name {profile} --endpoint-name {endpoint1}')
        self.cmd('cdn endpoint show -g {resource_group} '
                 '--profile-name {profile} '
                 '-n {endpoint1}',
                 checks=endpoint_checks)
        self.cmd('cdn endpoint waf policy remove -y -g {resource_group} '
                 '--profile-name {profile} '
                 '--endpoint-name {endpoint2}')
        with self.assertRaises(CLIError):
            self.cmd('cdn endpoint waf policy show -g {resource_group} --profile-name {profile} --endpoint-name {endpoint2}')
        self.cmd('cdn endpoint show -g {resource_group} '
                 '--profile-name {profile} '
                 '-n {endpoint2}',
                 checks=endpoint_checks)
        self.cmd('cdn waf policy show -g {resource_group} -n {policy}', checks=policy_checks)

    def endpoint_id(self, resource_group, profile, endpoint):
        return f'/subscriptions/{self.get_subscription_id()}' \
               f'/resourcegroups/{resource_group}' \
               f'/providers/Microsoft.Cdn' \
               f'/profiles/{profile}' \
               f'/endpoints/{endpoint}'

    def policy_id(self, resource_group, policy):
        return f'/subscriptions/{self.get_subscription_id()}' \
               f'/resourcegroups/{resource_group}' \
               f'/providers/Microsoft.Cdn' \
               f'/cdnwebapplicationfirewallpolicies/{policy}'


class CdnWafManagedRuleSetTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    def test_cdn_waf_managed_rule_set_crud(self, resource_group):

        rule_set_type = 'DefaultRuleSet'
        rule_set_version = '1.0'

        self.kwargs.update({
            'resource_group': resource_group,
            'rule_set_type': rule_set_type,
            'rule_set_version': rule_set_version,
        })

        checks = [JMESPathCheck('@[0].name', 'DefaultRuleSet_1.0')]
        self.cmd('cdn waf policy managed-rule-set list-available', checks=checks)

        checks = [JMESPathCheck('@[0].ruleGroupName', 'PROTOCOL-ATTACK'),
                  JMESPathCheck('length(@[0].rules)', 7)]
        self.cmd('cdn waf policy managed-rule-set rule-group-override list-available '
                 '--rule-set-type {rule_set_type} --rule-set-version {rule_set_version}',
                 checks=checks)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import tempfile
import unittest

from azure.cli.testsdk import ScenarioTest, LocalContextScenarioTest


class TestConfigure(unittest.TestCase):
    def test_configure_output_options(self):
        from azure.cli.core._output import AzOutputProducer
        from azure.cli.core.mock import DummyCli
        from azure.cli.command_modules.configure._consts import OUTPUT_LIST

        output_producer = AzOutputProducer(DummyCli())
        cli_output_options = set(output_producer._FORMAT_DICT.keys())
        configure_output_options = set(item["name"] for item in OUTPUT_LIST)

        self.assertEqual(cli_output_options, configure_output_options,
                         "\n{}'s output options: {}\ndon't match az configure's output options ({})."
                         .format(AzOutputProducer.__name__, cli_output_options, configure_output_options))


class ConfigureGlobalDefaultsTest(ScenarioTest):

    def setUp(self):
        self.local_dir = tempfile.mkdtemp()

    def tearDown(self):
        self.cmd('configure --defaults global="" global2=""')
        self.cmd('configure --defaults local="" --scope local')

    def test_configure_global_defaults(self):
        # setiing the az configure defaults
        self.cmd('configure --defaults global=global1')
        self.cmd('configure --defaults global2=global2')

        # configure a local setting
        os.chdir(self.local_dir)
        self.cmd('configure --defaults local=local1 --scope local')

        # test listing defaults
        res = self.cmd('configure --list-defaults --scope local').get_output_in_json()
        self.assertTrue(len(res), 3)
        actual = set([(x['name'], x['value']) for x in res])
        expected = set([('local', 'local1'), ('global', 'global1'), ('global2', 'global2')])
        self.assertTrue(actual == expected)
        res = self.cmd('configure --list-defaults').get_output_in_json()
        self.assertEqual(len(res), 2)
        actual = set([(x['name'], x['value']) for x in res])
        expected = set([('global', 'global1'), ('global2', 'global2')])
        self.assertTrue(actual == expected)


class LocalContextCommandsScenarioTest(LocalContextScenarioTest):

    def test_local_context_commands(self):
        self.cmd('local-context show')
        self.cmd('local-context show --name resource_group_name vnet_name')
        self.cmd('local-context delete --name resource_group_name vnet_name')
        self.cmd('local-context delete --all -y')
        self.cmd('local-context delete --all --purge -y')
        self.cmd('local-context delete --all --purge -y --recursive')

        from knack.util import CLIError
        with self.assertRaises(CLIError):
            self.cmd('local-context delete --name resource_group_name --all')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.util import sdk_no_wait


def create_log_analytics_workspace_linked_service(client, resource_group_name, workspace_name,
                                                  linked_service_name, resource_id=None, write_access_resource_id=None,
                                                  no_wait=False):
    from azure.mgmt.loganalytics.models import LinkedService

    linked_service = LinkedService(resource_id=resource_id,
                                   write_access_resource_id=write_access_resource_id)
    return sdk_no_wait(no_wait, client.create_or_update, resource_group_name,
                       workspace_name, linked_service_name, linked_service)


def update_log_analytics_workspace_linked_service(instance, resource_id=None, write_access_resource_id=None):
    if resource_id is not None:
        instance.resource_id = resource_id
    if write_access_resource_id is not None:
        instance.write_access_resource_id = write_access_resource_id
    return instance



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest
from azure_devtools.scenario_tests import AllowLargeResponse


class SecurityCenterDiscoveredSecuritySoluytionsTests(ScenarioTest):

    def test_security_discovered_security_solutions(self):

        discovered_security_solutions = self.cmd('az security discovered-security-solution list').get_output_in_json()

        assert len(discovered_security_solutions) >= 0
